#include <glib.h>
#include <GL/gl.h>



class Options
{
 public:
    Options ();
    ~Options ();
    /*
     * Read user options. If the user has not
     * specified some option, a default one are
     * used.
     *
     * This must be called before any other method.
     * On calling, message must be NULL (believe
     * me, this avoid memory leaks).
     * First we try to read a config file on ~/.3dfb
     * If it doesn't exists, we create one with default
     * values.
     * In future, we will also recognize command
     * arguments, environment variables and alternative
     * config files.
     *
     * On success, returns 0 and message is NULL.
     * Otherwise, returns -1 and message is an
     * allocated string that contains the error
     * explanation. This string must be freed
     * calling g_free(error_message).
     */
    int read_options (); // FIXME add error messaging

    /*
     * Option's accessors.
     */
    void get_file_color (GLfloat color[3]);
    void get_dir_color (GLfloat color[3]);
    void get_pdir_color (GLfloat color[3]);
    void get_fog_color (GLfloat color[4]);

    /*
     * Probably we will want to overload these.
     * i.e. symbolic color names could be used.
     */
    void set_file_color (const GLfloat color[3]);
    void set_dir_color (const GLfloat color[3]);
    void set_pdir_color (const GLfloat color[3]);
    void set_fog_color (const GLfloat color[4]);


 private:
    gchar *config_file;
    gboolean options_readed;
    GLfloat file_color[3];
    GLfloat dir_color[3];
    GLfloat pdir_color[3];
    GLfloat fog_color[4];

    void set_default_options ();
    /*
     * Save user options to persistent storage.
     *
     * Warning: if 'config_file' already exists, it is
     * overwritten! For this reason this is private now.
     * In future, we will merge actual options to the
     * existing config file and this function will be
     * public.
     *
     * error_message must be NULL.
     * 
     * Returns: same as read_options.
     */
    int save_options (); // FIXME add error messaging

    void create_new_config_file ();

    /*
     * For now auxiliary functions go here.
     * 
     * When the module became bigger, probably we will drop
     * out these as static functions outside any class, and
     * we will bring all together into a common namespace.
     */
    int parse_options (const gchar *buffer, GHashTable *table);
};

