#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

#include "files.hpp"


/*
 * FIXME this must be coordinated with
 * BUFSIZE on 3dfb.cpp, not a good practice
 */
#define BUFSIZE 256

/*
 * This is graphic info, must not be here
 */
#define max_size 5
#define file_div 1000


static long fileSize (const char* filename);
static tm* getMDate (const char* filename);
static mode_t getMMode (const char* filename);
static int isDir (const char* filename);



/*
 *
 * Public methods
 *
 */


fileMap::fileMap ()
{
    for (int i = 0; i < 256; i++)
        for (int j = 0; j < 256; j++)
            map[i][j].name = NULL;
    totalFiles = 0;
    numLevels = 0;
}


void fileMap::loadMap ()
{
    gchar* pch;
    gchar *cwd, *orig;
    int level, numFiles;
    int levelSize; // This is graphic info, must not be here
    DIR *dirp;
    struct dirent *dp;
    float localy;
    long fSize;
    gchar *error_msg;

    cwd = g_get_current_dir();
    orig = g_strdup(cwd);
    // FIXME This is for movement, mustn't go here!
    localy = 0;
    // the following if statement/for loop just sets the amount of levels
    level = 0;
    if (strlen(cwd) > 1) {
        pch = g_strstr_len (cwd, strlen(cwd), G_DIR_SEPARATOR_S);
        while (pch != NULL)
        {
            level++;
            pch = g_strstr_len (g_utf8_next_char(pch), strlen(cwd),
                                G_DIR_SEPARATOR_S);
        }
    }
    // <-- and now the number of levels is stored in numLevels
    numLevels = level;
    // the next function sets all the file information into the array
    totalFiles = 0;
    do
    {
        levelSize = 0;
        numFiles = 0;
        if ((dirp = opendir(".")) == NULL) {
            error_msg = g_strdup_printf("%s: could not open current directory",
                                        PACKAGE_NAME);
            perror(error_msg);
            g_free(error_msg);
            return;
        }
        do {
            if ((dp = readdir(dirp)) != NULL) {
                if (strcmp(dp->d_name, "."))
                {
                    fSize = fileSize(dp->d_name);
                    // This is graphic info, must not be here
                    if (fSize/file_div < max_size)
                    {
                        if (fSize/file_div > levelSize)
                            levelSize = fSize/file_div;
                    } else {
                        levelSize = max_size;
                    }
                    if (isDir(dp->d_name)) {
                        map[level][numFiles].is_dir = TRUE;
                        if (!strcmp(dp->d_name,"..")) {
                            map[level][numFiles].is_pdir = TRUE;
                        } else {
                            map[level][numFiles].is_pdir = FALSE;
                        }
                    } else {
                        map[level][numFiles].is_dir = FALSE;
                    }
                    setAll(level, numFiles, dp->d_name, fSize,
                           getMMode(dp->d_name), getMDate(dp->d_name));
                    setSelectInfo(totalFiles, level, numFiles);
                    numFiles++;
                    totalFiles++;
                }
            }
        } while (dp != NULL && numFiles < (BUFSIZE - 1));
        closedir (dirp);
        g_free(cwd);
        cwd = g_get_current_dir();
        setLevelData(level, cwd, numFiles, levelSize);
        localy += levelSize + 1;
        chdir("..");
        level--;
    } while (level > -1);
    // <-- keep going until you are at the root directory
    *y = localy;
    chdir(orig); // go back to the original directory
    g_free(orig);
    g_free(cwd);
    return;   
}


gchar *fileMap::getName (unsigned int level, unsigned int number)
{
    g_assert(level < 256);
    g_assert(level < 256);
    return (g_strdup(map[level][number].name));
}


unsigned int fileMap::getNumFiles ()
{
    return totalFiles;
}


unsigned int fileMap::getNumLevels ()
{
    return numLevels;
}


long fileMap::size (unsigned int level, unsigned int number)
{
    return map[level][number].size;
}


mode_t fileMap::getPerms (unsigned int level, unsigned int number)
{
    g_assert(level < 256);
    g_assert(level < 256);
    return map[level][number].permissions;
}


tm* fileMap::getDate (unsigned int level, unsigned int number)
{
    g_assert(level < 256);
    g_assert(level < 256);
    return map[level][number].m_date;
}


gboolean fileMap::is_dir (unsigned int level, unsigned int number)
{
    g_assert(level < 256);
    g_assert(level < 256);
    return (map[level][number].is_dir);
}


gboolean fileMap::is_pdir (unsigned int level, unsigned int number)
{
    g_assert(level < 256);
    g_assert(level < 256);
    return (map[level][number].is_pdir);
}


gboolean fileMap::isExecutable (unsigned int level, unsigned int number)
{
    mode_t mode;

    mode = getPerms (level, number);

    return ((mode & S_IXUSR) || (mode & S_IXGRP) || (mode & S_IXOTH));
}


int fileMap::execute (unsigned int level, unsigned int number)
{
    gchar *name;
    gchar *filePath;
    pid_t child;

    g_assert(level < 256);
    g_assert(level < 256);

    name = getName (level, number);
    filePath = g_strconcat (getLevelPath(level), G_DIR_SEPARATOR_S, name, 0);
    g_free (name);

    child = 1;
    if (isExecutable (level, number)) {
        child = fork ();
        if (child == 0) {
            exit (system (filePath));
        }
    } else {
        g_message ("File %s is not executable", filePath);
    }

    g_free (filePath);

    /*
     * Probably we must keep the pid_t on a table, and
     * when exit just kill them, to avoid zoombies.
     */
    return (child > 0 ? 0 : -1);
}

gchar* fileMap::getLevelPath (unsigned int level)
{
    return levels[level].path;
}

int fileMap::getLevelNumFiles (unsigned int level)
{
    return levels[level].numFiles;
}

int fileMap::getLevelSize (unsigned int level)
{
    return levels[level].levelSize;
}


/*
 *
 * Private methods
 *
 */


void fileMap::setAll (unsigned int level, unsigned int number,
                      char* n, long s, mode_t p, tm* md)
{
    g_assert(level < 256);
    g_assert(level < 256);
    g_assert(n != NULL);
    if (map[level][number].name)
        g_free(map[level][number].name);
    map[level][number].name = g_strdup(n);
    map[level][number].size = s;
    map[level][number].permissions = p;
    map[level][number].m_date = md;
}


void fileMap::setLevelData (unsigned int level, char* fn, int nf, int ls)
{
    g_assert(level < 1024);
    g_assert(fn);
    levels[level].path = g_strdup(fn);
    levels[level].numFiles = nf;
    levels[level].levelSize = ls;
}


void fileMap::setSelectInfo (unsigned int which, int level, int filenum)
{
    g_assert(which < 1024);
    g_assert(level < 256);
    g_assert(filenum < 256);
    fileNames[which].level = level;
    fileNames[which].filenum = filenum;
}


/*
 *
 * Auxiliary functions
 *
 */


static long fileSize(const char* filename)
{
   struct stat info;

   if(lstat(filename, &info) == 0)
       return info.st_size;
   //perror("fileSize(): lstat failed");
    return 0;
}

static tm* getMDate(const char* filename)
{
    struct stat info;

    if(lstat(filename, &info) == 0)
        return localtime(&info.st_mtime);
    //perror("getMDate(): lstat failed");
    return NULL;
}

static mode_t getMMode(const char* filename)
{
    struct stat info;

    if(lstat(filename, &info) == 0)
        return info.st_mode;
    //perror("getMMode(): lstat failed");
    return 0;
}

static int isDir(const char* filename)
{
    struct stat info;

    if(lstat(filename, &info) == 0){
        if (S_ISDIR(info.st_mode)){
            return 1;
        }
    }
    //perror("isDir(): lstat failed");
    return 0;
}
