#include <dirent.h>
#include <sys/stat.h>
#include <time.h>
#include <glib.h>



typedef struct {

    gchar *path;	// absolute path to the level
    int numFiles;	// number of files in the level
    int levelSize;	// calculated from the largest file in the level

} levelData;


typedef struct {

    unsigned int level;
    unsigned int filenum;

} selectList;


typedef struct {

    gchar *name;
    long size;
    mode_t permissions;
    tm* m_date; // modification date
    gboolean is_dir;
    gboolean is_pdir;

} fileData;


class fileMap
{
 public:

    fileMap();

    /*
     * Maximum number of levels on the fileMap. Useful for 
     * allocate static arrays.
     */
    static const unsigned int maxLevels = 1024;

    /*
     * Point to the y axis control variable. Must be
     * filled before to call initMap.
     *
     * This is for movement, and mustn't go here, is
     * a terrible hack!! Will be fixed soon.
     */
    float *y;

    /*
     * Read the entire file tree, and sets the
     * information for every file.
     *
     * This must be called before any other method.
     */
    void loadMap (void);

    /*
     * Retuns the total files number.
     */
    unsigned int getNumFiles (void);

    /*
     * Retuns the total levels number.
     */
    unsigned int getNumLevels (void);
    
    /*
     * Returns a copy of the name of the file 'number'
     * on level 'level'
     *
     * The returning string must be freed with g_free()
     */
    gchar *getName (unsigned int level, unsigned int number);
    
    /*
     * Size of file 'number' on level 'level', in bytes.
     */
    long size (unsigned int level, unsigned int number);

    mode_t getPerms (unsigned int level, unsigned int number);

    tm* getDate (unsigned int level, unsigned int number);
        
    /*
     * TRUE if the file is a directory (parent or not)
     */
    gboolean is_dir (unsigned int level, unsigned int number);
    
    /*
     * TRUE if the file is a parent directory
     */
    gboolean is_pdir (unsigned int level, unsigned int number);

    /*
     * TRUE if the file is executable.
     *
     * If TRUE is returned, that doesn't mean that the current user
     * can execute the file, only that somebody can execute the file.
     */
    gboolean isExecutable (unsigned int level, unsigned int number);

    /*
     * Executes the file 'number' at level 'level'
     *
     * For now we only try to execute, in future
     * we will also launch other applications for 
     * well-known file extensions.
     *
     * Returns 0 on success, -1 on failure.
     */
    int execute (unsigned int level, unsigned int number);

    /*
     * Returns the absolute path to level 'level', or NULL on error.
     *
     * If the string should be modified, must be copied away with
     * g_strdup().
     */
    gchar *getLevelPath (unsigned int level);

    /*
     * Returns the number of files on the given level, or -1
     * on error.
     */
    int getLevelNumFiles (unsigned int level);

    /*
     * Returns the Size of the level, based on the largest
     * file in the level.
     */
    int getLevelSize (unsigned int level);

    /*
     * In future, this must be a private field
     */
    selectList fileNames[maxLevels];

    
 private:

    /*
     * If we have more than 256 entries on a directory, we have 
     * a problem, and if we have less than 256 entries, there is
     * a waste of memory. In future this must be dynamic.
     */
    fileData map[256][256];

    levelData levels[maxLevels];

    unsigned int totalFiles;
    unsigned int numLevels;

    void setAll (unsigned int level, unsigned int number,
                 char* n, long s, mode_t p, tm* md);
    void setLevelData (unsigned int level, char* fn,int nf,int ls);
    void setSelectInfo (unsigned int which, int level, int filenum);
};



